
--------------------------------------------------------------------------------
--helper function : custom status message --prefixes tool name and adds brackets 
--------------------------------------------------------------------------------
local function status(message)
  renoise.app():show_status("Mix Filter Cycle Tool: ("..message..")")
end

--global filter counter
local filter_count = 1

--default Reference Filter presets used by [Low][Mid][High] focus buttons
local low_filter_preset = 
[[
<?xml version="1.0" encoding="UTF-8"?>
<FilterDevicePreset doc_version="11">
  <DeviceSlot type="DigitalFilterDevice">
    <IsMaximized>true</IsMaximized>
    <OversamplingFactor>2x</OversamplingFactor>
    <Model>Chebyshev 8n</Model>
    <Type>
      <Value>0.0</Value>
    </Type>
    <Cutoff>
      <Value>0.125</Value>
    </Cutoff>
    <Q>
      <Value>0.0</Value>
    </Q>
    <Ripple>
      <Value>0.0</Value>
    </Ripple>
    <Inertia>
      <Value>0.0078125</Value>
    </Inertia>
    <ShowResponseView>true</ShowResponseView>
    <ResponseViewMaxGain>18</ResponseViewMaxGain>
  </DeviceSlot>
</FilterDevicePreset>]]

local mid_filter_preset = 
[[
<?xml version="1.0" encoding="UTF-8"?>
<FilterDevicePreset doc_version="11">
  <DeviceSlot type="DigitalFilterDevice">
    <IsMaximized>true</IsMaximized>
    <OversamplingFactor>2x</OversamplingFactor>
    <Model>Chebyshev 8n</Model>
    <Type>
      <Value>1.0</Value>
    </Type>
    <Cutoff>
      <Value>0.35</Value>
    </Cutoff>
    <Q>
      <Value>0.227999985</Value>
    </Q>
    <Ripple>
      <Value>0.0</Value>
    </Ripple>
    <Inertia>
      <Value>0.0078125</Value>
    </Inertia>
    <ShowResponseView>true</ShowResponseView>
    <ResponseViewMaxGain>18</ResponseViewMaxGain>
  </DeviceSlot>
</FilterDevicePreset>]]

--[[
local high_filter_preset = 
[-[
<?xml version="1.0" encoding="UTF-8"?>
<FilterDevicePreset doc_version="11">
  <DeviceSlot type="DigitalFilterDevice">
    <IsMaximized>true</IsMaximized>
    <OversamplingFactor>2x</OversamplingFactor>
    <Model>Chebyshev 8n</Model>
    <Type>
      <Value>3</Value>
    </Type>
    <Cutoff>
      <Value>0.50</Value>
    </Cutoff>
    <Q>
      <Value>0.227999985</Value>
    </Q>
    <Ripple>
      <Value>0.0</Value>
    </Ripple>
    <Inertia>
      <Value>0.0078125</Value>
    </Inertia>
    <ShowResponseView>true</ShowResponseView>
    <ResponseViewMaxGain>18</ResponseViewMaxGain>
  </DeviceSlot>
</FilterDevicePreset>]-]

--]]

local high_filter_preset = 
[[
<?xml version="1.0" encoding="UTF-8"?>
<FilterDevicePreset doc_version="12">
  <DeviceSlot type="DigitalFilterDevice">
    <IsMaximized>true</IsMaximized>
    <OversamplingFactor>2x</OversamplingFactor>
    <Model>Chebyshev 8n</Model>
    <Type>
      <Value>3</Value>
    </Type>
    <Cutoff>
      <Value>0.293999881</Value>
    </Cutoff>
    <Q>
      <Value>0.227999985</Value>
    </Q>
    <Ripple>
      <Value>0.0</Value>
    </Ripple>
    <Inertia>
      <Value>0.0078125</Value>
    </Inertia>
    <ShowResponseView>true</ShowResponseView>
    <ResponseViewMaxGain>18</ResponseViewMaxGain>
  </DeviceSlot>
</FilterDevicePreset>
]]

----------------------------------------------------------------------
----------------------------------------------------------------------
function main_mix_fil_cycle()
----------------------------------------------------------------------
----------------------------------------------------------------------
  local song = renoise.song()
  local master_index = #song.tracks - song.send_track_count
  local master_track = song:track(master_index)
  
  
  local mas_ref_filter = nil
  --2) loop backwards through master track devices to find filter
  for devs = 2, #master_track.devices do
    --find toggle gainer
    if master_track.devices[devs].display_name == "'MIB' Cycle Filter" then 
      mas_ref_filter = master_track.devices[devs]
      break
    end
  end
  
  --add master filter if not present
   --add filter at end of ref track if not present
  if mas_ref_filter == nil then
    mas_ref_filter = master_track:insert_device_at("Audio/Effects/Native/Digital Filter", #master_track.devices + 1)
    mas_ref_filter.display_name = "'MIB' Cycle Filter"
    mas_ref_filter.is_active = false --will toggle back on next
    filter_count = 0
  end
  
   --put filter at end of chain
   ----------------------------------
   --loop devices
   for device = 2,#master_track.devices do
    -- print(master_track.devices[device+1].display_name)
 
   
      if master_track.devices[device].display_name == "'MIB' Cycle Filter" then
      
        --get second last device position index
        local second_last_pos = (#master_track.devices -1)

        for pos = device,second_last_pos do
          --break if the last device is Gliss EQ --(always wanted last in master track)
          if master_track.devices[pos+1].display_name == "VST: Voxengo: GlissEQ2" then
           -- print("hi")
            break
          end
          --swap device
          master_track:swap_devices_at(pos, pos+1)
        end 
      end 
   end
  
  
  --[[-----------------------------------NEW
  --toggle different filter ranges
  if mas_ref_filter.is_active == false then
    mas_ref_filter.is_active = true
    --engage low preset
    mas_ref_filter.active_preset_data = high_filter_preset
    --increment filter count
    filter_count = filter_count + 1
    --status
    status("Global filter: HIGH: 0.6KHz- 20KHz")

  else --disable filter
    mas_ref_filter.is_active = false 
    filter_count = 0 --reset counter
    --status
    status("Global filter: OFF")
  end  --NEW--]]
 
 
 

  if filter_count == 0 then --engage low filter
    mas_ref_filter.is_active = true
    --engage low preset
    mas_ref_filter.active_preset_data = low_filter_preset
    --increment filter count
    filter_count = filter_count + 1
    --status
    status("Global filter: LOW: 0 - 200 Hz")
  
  elseif filter_count == 1 then --engage mid filter
    mas_ref_filter.is_active = true
    --engage low preset
    mas_ref_filter.active_preset_data = mid_filter_preset
    --increment filter count
    filter_count = filter_count + 1
    --status
    status("Global filter: MID: 200 - 2000 Hz")
  
  elseif filter_count == 2 then --engage high filter
    mas_ref_filter.is_active = true
    --engage low preset
    mas_ref_filter.active_preset_data = high_filter_preset
    --increment filter count
    filter_count = filter_count + 1
    --status
    status("Global filter: HIGH: 2KHz- 20KHz")
  
  else --disable filter
    mas_ref_filter.is_active = false 
    filter_count = 0 --reset counter
    --status
    status("Global filter: OFF")
    
  end  
  
end
